@@ -1,5 +1,5 @@
 /* internal.h --- Internal header file for GSS.
- * Copyright (C) 2003-2011 Simon Josefsson
+ * Copyright (C) 2003-2014 Simon Josefsson
  *
  * This file is part of the Generic Security Service (GSS).
  *
@@ -71,5 +71,9 @@ _gss_encapsulate_token_prefix (const char *prefix, size_t prefixlen,
 			       const char *in, size_t inlen,
 			       const char *oid, OM_uint32 oidlen,
 			       void **out, size_t * outlen);
+extern int
+_gss_decapsulate_token (const char *in, size_t inlen,
+			char **oid, size_t * oidlen,
+			char **out, size_t * outlen);
 
 #endif /* _INTERNAL_H */
