@@ -1,5 +1,5 @@
 /* krb5/context.c --- Implementation of Kerberos 5 GSS Context functions.
- * Copyright (C) 2003-2011 Simon Josefsson
+ * Copyright (C) 2003-2014 Simon Josefsson
  *
  * This file is part of the Generic Security Service (GSS).
  *
@@ -213,6 +213,11 @@ gss_krb5_init_sec_context (OM_uint32 * minor_status,
       return GSS_S_NO_CRED;
     }
 
+  if (target_name == NULL)
+    {
+      return GSS_S_BAD_NAME;
+    }
+
   if (k5 == NULL)
     {
       k5 = ctx->krb5 = calloc (sizeof (*k5), 1);
